<?php
namespace PAE\Strategies;

use PAE\Services\MarcaService as MarcaService;
use Teknisa\Libs\Util\Environment;

class MarcaStrategy implements Interfaces\IStrategy
{
    protected $marcaService;
    protected $errors = array();
    
    public function __construct(MarcaService $marcaService, Environment $environment)
    {
		$this->marcaService = $marcaService;
		$this->environment = $environment;
	}
	
	public function clearErrors()
	{
	    $this->errors = [];
	}
	
	public function formatCdMarca($cdMarca)
	{
	    return str_pad($cdMarca, 5, "0", STR_PAD_LEFT);
	}
	
	public function setError($columnName, $message)
	{
        array_push(
            $this->errors, 
            array(
                "COLUMN" => $columnName,
                "DESCERROR" => $message
            )
        );
    }
    
    public function validate(&$data)
    {
        foreach ($data as &$row) {
            self::verifyIsEmpty($row, ['CDMARCA', 'NMMARCA', 'IDMARCAATIVA']);
            
            self::verifyCdMarca($row);
            
            if ($row['IDMARCAATIVA'] != 'A' && $row['IDMARCAATIVA'] != 'I') 
                self::setError('IDMARCAATIVA', "somente aceita os valores 'A' ou 'I'");
            
            $row['ERROS'] = $this->errors;
            
            self::clearErrors();
        }
    }
    
    private function verifyIsEmpty($row, $values)
    {
        foreach ($values as $value) {
            if (empty($row[$value])) {
                self::setError($value, "não informado");
            } 
        }
    }
    
    private function verifyCdMarca($row)
    {
        if (!ctype_digit(strval($row['CDMARCA']))) {
            self::setError('CDMARCA', "deve conter somente digitos numéricos");
        }
        
        if (strlen($row['CDMARCA']) > 5) {
            self::setError('CDMARCA', "inválido, limite de 5 números ultrapassado");
        }
        
        $row['CDMARCA'] = self::formatCdMarca($row['CDMARCA']);
        
        if ($this->marcaService->verifyMarcaExists($row)) {
            self::setError('CDMARCA', "já existe");
        }
    }
    
    public function formatToView(&$data)
    {
        foreach ($data as &$row) {
           $row['CDMARCA'] = self::formatCdMarca($row['CDMARCA']);
           $row['IDERROR'] = !empty($row['ERROS']) ? 'S' : 'N';
        }
    }
    
    public function save($data)
    {
        self::validate($data);
        
        self::verifyExistErros($data);
        
        $this->marcaService->saveImport($data);
    }
    
    public function verifyExistErros($data)
    {
        foreach ($data as $row) {
            if (!empty($row['ERROS']))
                throw new \Exception('Dados com erros. Não foi possível efetuar salvamento.');
        } 
    }
    
    public function formatToSave(&$data)
    {
        foreach ($data as &$row) {
            $row['__is_new'] = true;
            $row['NRORG'] = $this->environment->getNrorg();
            $row['NRORGINCLUSAO'] = $this->environment->getNrorg();
            $row['NRORGULTATU'] = $this->environment->getNrorg();
            $row['CDOPERINCLUSAO'] = $this->environment->getCdOperador();
            $row['CDOPERULTATU'] = $this->environment->getCdOperador();
            $row['IDATIVO'] = 'S';
        } 
    }
}
